/*
 * libpcap-style tokens for building custom filters.
 *
 * (C) 2011 Florian Westphal <fw@strlen.de>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 2 of the License, or
 *      (at your option) any later version.
 */
%{
#include <stdio.h>
#include "parser_common.h"
#include "bsf_filter.yy.h"

extern void parse_bsf_string(const char *bsf);

static struct filter_expr_state filter_state;
%}

digits		[0-9]+

%%
src			filter_state.direction = SRC; return SRC;
dst			filter_state.direction = DST; return DST;
not			return NOT;
or			return OP_OR;
proto			return L4PROTO;
{digits}		yylval.number=atoi(yytext); return DIGIT;
[a-zA-Z]+		yylval.str = strdup(yytext); return WORD;
[a-fA-F0-9\:]+		yylval.str = strdup(yytext); return V6ADDR;
{digits}\.{digits}\.{digits}\.{digits} yylval.str = strdup(yytext); return V4ADDR;
\/			return SLASH;
\!                      return NOT;
[ \t\n]+                  /* whitespace */;
%%
void parse_bsf_string(const char *bsf)
{
	YY_BUFFER_STATE bufferstate = yy_scan_string(bsf);

	memset(&filter_state, 0, sizeof(filter_state));
	yyparse(&filter_state);
	yy_delete_buffer(bufferstate);
}

int yywrap()
{
        return 1;
}

