#include <stdbool.h>
#include <glib.h>

#include <libnetfilter_conntrack/libnetfilter_conntrack.h>


/**
 * @file ct_acct.h
 * group accounting tracking
 */

/**
 * @struct ct_acct
 * track accounting group statistics.
 * This structure should not be accessed directly, use
 * the getter functions instead.
 */
struct ct_acct {
	unsigned int flows; /*!< flows number of flows in the accouting group */
	GHashTable *src_addrs; /*!< src_addrs table of known source addresses */
	GHashTable *conntracks; /*!< conntracks that are part of this group */
};

/**
 * @param acct accounging group to inspect
 * @return number of flows in acct group
 */
static inline unsigned int ct_acct_get_flow_count(const struct ct_acct *acct)
{
	return acct->flows;
}

/**
 * @param acct accounging group to inspect
 * @return number of distinct source addresses in acct group
 */
static inline unsigned int ct_acct_get_saddr_count(const struct ct_acct *acct)
{
	return g_hash_table_size(acct->src_addrs);
}

bool ct_acct_del(const struct nf_conntrack *ct, struct ct_acct *acct);
void ct_acct_add(const struct nf_conntrack *ct, struct ct_acct *acct);

bool ct_acct_resync(struct nf_conntrack *ct, struct ct_acct *acct);

void ct_acct_init(const struct nf_conntrack *ct, struct ct_acct *acct, unsigned int flags);
